/*
 *    Copyright (c) 2016 by  Maciej Wiecierzewski
 */

var GameState = function(game)
{
    this.score = 0;
    this.points = 7;    // amout of points bubble bursting scores
    this.maxScore = null;
    this.currentSound = null;   // current player sound

    // stores data for player's bouncing back mechanism
    this.hedgehogCollisionObject = { count: 0, posChange: 0, side: '' };

    /*
        The player is represented by two sprites: an actual displayed sprite,
        and a body sprite for physics. Phaser does not detect new collision
        between objects if their sprites are still overlapping, even if their
        bodies had gotten separated.

        The player's sprite position is updated in preRender() function.
    */
    this.player = {
        sprite: null,
        bodySprite: null,

        direction: 'right',
        onGround: false,
        walking: false,

        // used for interacting with clouds
        locked: false,      // is the player locked to a cloud
        lockedTo: null,     // cloud's Phaser.Sprite
        lockedToX: 0        // x position of the cloud; for calculating vertical position change
    };

    // sprite groups
    this.platforms = null;
    this.grassBack = null;
    this.grassFront = null;
    this.clouds = null;
    this.hedgehogs = null;
    this.bubbles = null;

    // interface elements
    this.scoreText = null;
    this.soundButton = null;
}

//------------------------------------------------------------------------------
// Creating functions
//------------------------------------------------------------------------------

GameState.prototype.createPlayer = function()
{
    var game = this.game;

    //creating player's sprite
    var playerSprite = game.add.sprite(100, 950, 'woofy');

    playerSprite.anchor.setTo(0.5, 0);

    // setting up animation
    var walkFrames = Phaser.Animation.generateFrameNames('walk', 0, 29);
    var walkEndLeftFrames = Phaser.Animation.generateFrameNames('walk_end_left', 0, 23);
    var walkEndRightFrames = Phaser.Animation.generateFrameNames('walk_end_right', 0, 23);
    var walkStartFrames = Phaser.Animation.generateFrameNames('walk_start', 0, 2);
    var jumpBeginFrames = Phaser.Animation.generateFrameNames('jump_begin', 0, 13);
    var jumpEndFrames = Phaser.Animation.generateFrameNames('jump_end', 14, 41);
    playerSprite.animations.add('walk', walkFrames);
    playerSprite.animations.add('walk_end_left', walkEndLeftFrames);
    playerSprite.animations.add('walk_end_right', walkEndRightFrames);
    playerSprite.animations.add('walk_start', walkStartFrames);
    playerSprite.animations.add('jump_begin', jumpBeginFrames);
    playerSprite.animations.add('jump_end', jumpEndFrames);

    this.player.animation = new PlayerAnimation(playerSprite);
    this.player.animation.onWalkBegin = function(){ this.playWalkSound(); }.bind(this);
    this.player.animation.onWalkEnd = function(){ this.stopWalkSound(); }.bind(this);

    // makes game.physics.arcade.overlap() work
    game.physics.arcade.enable(playerSprite);

    this.player.sprite = playerSprite;

    // creating player's bodySprite
    var width = 45;
    var height = 110;

    var bmd = game.add.bitmapData(width, height);
    /*bmd.ctx.beginPath();
    bmd.ctx.rect(0, 0, width, height);
    bmd.ctx.fillStyle = '#fff';
    bmd.ctx.fill();*/

    var playerBody = game.add.sprite(100, 950, bmd);
    game.physics.arcade.enable(playerBody);

    playerBody.body.gravity.y = 800;
    playerBody.body.checkCollision.up = false;
    playerBody.body.collideWorldBounds = true;
    playerBody.anchor.setTo(0.5, 0);

    this.player.bodySprite = playerBody;
}

//------------------------------------------------------------------------------

GameState.prototype.createGround = function(dataArr)
{
    var game = this.game;

    for(var i = 0; i < dataArr.length; i++)
    {
        var entry = dataArr[i];

        // grass top layer
        var grassFront = game.add.tileSprite(entry.x, entry.y-20, entry.length, 20, 'grass_tile_top');
        var grassLeft = game.add.sprite(entry.x-10, entry.y-20, 'grass_left');
        var grassRight = game.add.sprite(entry.x+entry.length-10, entry.y-20, 'grass_right');
        this.grassFront.add(grassFront);
        this.grassFront.add(grassLeft);
        this.grassFront.add(grassRight);

        var grassBack = game.add.tileSprite(entry.x, entry.y-20, entry.length, 20, 'grass_tile_top2');
        this.grassBack.add(grassBack);

        // main sprite
        var ground = game.add.tileSprite(entry.x, entry.y, entry.length, 55, 'grass_tile');
        game.physics.arcade.enable(ground);
        ground.body.immovable = true;
        this.platforms.add(ground);

        // ground fill beneath grass
        var height = game.world.bounds.height - entry.y;
        var groundTile = game.add.tileSprite(entry.x, entry.y, entry.length, height, 'ground_tile');

        this.ground.add(groundTile);

        // ground edges
        if(entry.edgeDepth > 0)
        {
            var groundLeft = game.add.tileSprite(entry.x-10, entry.y, 15, entry.edgeDepth, 'ground_left');
            this.ground.add(groundLeft);

            var groundRight = game.add.tileSprite(entry.x+entry.length-10, entry.y+50, 15, entry.edgeDepth, 'ground_right');
            this.ground.add(groundRight);
        }
    }
}

//------------------------------------------------------------------------------

GameState.prototype.createLedges = function(dataArr)
{
    var game = this.game;
    var platforms = this.platforms;

    for(var i = 0; i < dataArr.length; i++)
    {
        var entry = dataArr[i];

        var grassFront = game.add.tileSprite(entry.x, entry.y-20, entry.length, 20, 'grass_tile_top');
        var grassLeft = game.add.sprite(entry.x-10, entry.y-20, 'grass_left');
        var grassRight = game.add.sprite(entry.x+entry.length-10, entry.y-20, 'grass_right');
        this.grassFront.add(grassFront);
        this.grassFront.add(grassLeft);
        this.grassFront.add(grassRight);

        var grassBack = game.add.tileSprite(entry.x, entry.y-20, entry.length, 20, 'grass_tile_top2');
        this.grassBack.add(grassBack);

        // main sprite
        var ledge = game.add.tileSprite(entry.x, entry.y, entry.length, 55, 'grass_tile');
        game.physics.arcade.enable(ledge);
        ledge.body.immovable = true;
        ledge.body.checkCollision.left = false;
        ledge.body.checkCollision.right = false;

        platforms.add(ledge);
    }
}

//------------------------------------------------------------------------------

GameState.prototype.createClouds = function(dataArr)
{
    var game = this.game;
    var clouds = this.clouds;
    var front = this.cloudsFront;
    var back = this.cloudsBack;

    for(var i = 0; i < dataArr.length; i++)
    {
        var entry = dataArr[i];

        var cloud = new CloudPlatform(entry.x, entry.y, entry.length, 10, game, clouds, back, front);
        cloud.setMotionPath(entry.motionPath, 2000);
        cloud.start();
    }
}

//------------------------------------------------------------------------------

GameState.prototype.createHedgehogs = function(dataArr)
{
    var game = this.game;
    var hedgehogs = this.hedgehogs;

    for(var i = 0; i < dataArr.length; i++)
    {
        var entry = dataArr[i];

        var hedghehog = new Hedgehog(entry.xMin, entry.xMax, entry.y, entry.speed, game, hedgehogs);
    }
}

//------------------------------------------------------------------------------

GameState.prototype.createBubbles = function(dataArr)
{
    var game = this.game;
    var bubbles = this.bubbles;

    for(var i = 0; i < dataArr.length; i++)
    {
        var entry = dataArr[i];

        var key = "";
        if(entry.size === 'bg')
            key = 'bubble_bg';
        else if(entry.size === 'md')
            key = 'bubble_md';
        else if(entry.size === 'sm')
            key = 'bubble_sm';

        var bubble = bubbles.create(entry.x, entry.y-20, key);
        bubble.animations.add('burst');
    }
}

//------------------------------------------------------------------------------
// Collisions
//------------------------------------------------------------------------------

GameState.prototype.burstBubble = function(player, bubble)
{
    this.bubbles.remove(bubble);
    this.burstedBubbles.add(bubble);
    bubble.animations.play('burst', 60, false, true);

    // scoring points
    this.score += this.points;
    this.updateTexts();

    // sound
    var i = Math.round(Math.random()*5)+1;
    this.sounds['pop'+i].play();
}

//------------------------------------------------------------------------------

GameState.prototype.hedgehogCollision = function(player, hedgehogSprite)
{
    var hedgehog = hedgehogSprite.hedgehog;

    var collObj = this.hedgehogCollisionObject;

    if(player.body.touching.down && collObj.count < 2)
        player.body.velocity.y = -300;

    if(hedgehogSprite.body.x < player.body.x)   // if player collided on his left side
    {
        player.body.velocity.x = 30;
        collObj.posChange = 1;
        collObj.side = 'left';

        if(hedgehog.direction === 'right')
            hedgehog.reverse();
    }
    else if(hedgehogSprite.body.x > player.body.x)  // if player collided on his right side
    {
        player.body.velocity.x = -30;
        collObj.posChange = -1;
        collObj.side = 'right';

        if(hedgehog.direction === 'left')
            hedgehog.reverse();
    }

    collObj.count = 45;

    // ouch sound
    if(this.currentSound !== null && this.currentSound.isPlaying)
        return;

    var i = Math.round(Math.random()*6)+1;
    this.sounds['ouch'+i].play();
    this.currentSound = this.sounds['ouch'+i];
}

//------------------------------------------------------------------------------

GameState.prototype.platformCollision = function(player, platform)
{
    if(player.body.touching.down && this.player.onGround === false)
    {
        this.player.animation.hitGround();

        this.sounds['landing'].play();

        this.player.onGround = true;
    }
}

//------------------------------------------------------------------------------

GameState.prototype.cloudCollision = function(playerBody, cloud)
{
    var player = this.player;

    if(!player.locked && playerBody.body.velocity.y > 0)
    {
        player.locked = true;
        player.lockedTo = cloud;
        player.lockedToX = cloud.x;

        playerBody.body.velocity.y = 0;

        player.animation.hitGround();
    }
}

//------------------------------------------------------------------------------

//  used for locking the player on a cloud
GameState.prototype.checkLock = function()
{
    var player = this.player;
    var playerBody = this.player.bodySprite;

    if((playerBody.body.right < player.lockedTo.body.x || playerBody.body.x > player.lockedTo.body.right)
            && playerBody.body.y - player.lockedTo.body.y < 20)
    {
        return false;
    }
    else
        return true;
}

//------------------------------------------------------------------------------

GameState.prototype.cancelLock = function()
{
    this.player.locked = false;
    this.player.lockedTo = null;
}

//------------------------------------------------------------------------------
// miscellaneous interface related functions
//------------------------------------------------------------------------------

GameState.prototype.updateTexts = function()
{
    this.scoreText.text = "score: " + this.score;

    if(this.score === this.maxScore)
    {
        this.endText.text = "All bubbles burst.";
    }
}

//------------------------------------------------------------------------------

GameState.prototype.toggleSound = function()
{
    if(this.soundButton.frame === 0)
    {
        this.soundButton.frame = 1;
        this.game.sound.mute = true;
    }
    else
    {
        this.soundButton.frame = 0;
        this.game.sound.mute = false;
    }
}

//------------------------------------------------------------------------------

GameState.prototype.playWalkSound = function()
{
    if(this.player.locked)  // no sound on clouds
        return;

    this.sounds['steps'].play("", 0, 1, true);
}

//------------------------------------------------------------------------------

GameState.prototype.stopWalkSound = function()
{
    if(this.player.locked)  // no sound on clouds
        return;

    if(this.player.animation.state === PlayerAnimation.States.END_WALK)
        this.sounds['steps'].play("", 0, 1, false);     // plays last step sound
    else
        this.sounds['steps'].stop();
}
